"use strict";
const entities = require('entities');
const xpath = require('xpath');
const path = require('path');
const moment = require('moment-timezone');
const dom = require('xmldom').DOMParser;
const Parser = require('./Parser.js');

module.exports = class ParserColumn extends Parser{

    constructor(config, key) {

        super();

        this.config = config;
        this.key = key;


    }
    
    /**
     *
     * Parse a multi-block fixed-width. Lines go in, blocks with items out.
     * 
     * @param asc - String
     *
     * 
     */
    parseMultiBlockFW(lines, filename) {

        this.filename = filename;
        this.globalDate = this.getGlobalDate();

        var blocks = {};

        var skip = 0;
        if (typeof this.config.skip_lines != undefined &&  this.config.skip_lines) {
            skip = this.config.skip_lines;
        }

        if(typeof this.config.asc_column_width == "undefined" || this.config.asc_column_width.length < 1) {
            throw Error("Fixed width columns width not defined, cannot parse anything.")
        }

        
        for(var i=skip; i < lines.length; i++) {

            var line = lines[i];
            if (line.trim() == '') {
                continue;
            }

            var row = {}
            var lastIndex = 0;

            //loop through all column definitions. First make a row variable containing the data per row number
            for (var index in this.config.asc_column_width) {
                var columnWidth = this.config.asc_column_width[index];
                var columnEnd = lastIndex + columnWidth * 1; //*1 to parse as int...
                row[index] = line.substring(lastIndex, columnEnd).trim(); //trim, very important!
                lastIndex = columnEnd;
            }

            //find the model_type in order to succesfully parse the rest of the data
            var item = this.parseItemColumned(row);

            if (!item) {
                continue;
            }

            if (typeof this.config.multi_block == "undefined" || this.config.multi_block != 'key') {
                //this is a playlist-day, but no multi-block block key has been defined
                //the parser has saved the hour from the raw date string as block identifier
                var block_id = item['_rawhour_block']
                delete item['_rawhour_block'];
            } else {
                var block_id = item[this.config.blocks_key];
            }

            if (typeof blocks[block_id] == 'undefined') {
                let startTime = this.globalDate.clone();
                startTime.add(block_id, 'hours');
                blocks[block_id] = {'start': startTime.format(), 'items': []};
            }

            blocks[block_id].items.push(item);


        }

        //fix the durations afterwards (if not present), less complicated
        for (var i in blocks) {
            let items = blocks[i].items;
            for (var j in items) {
                if (items[j].duration < 0) {
                    items[j].duration = 10; //default duration
                    if (j > 0) {
                        items[j - 1].duration = items[j]._start_seconds - items[j - 1]._start_seconds;
                    }
                }
            }
            for (var j in items) {
                delete items[j]['_start_seconds'];
                delete items[j]['start'];
            }

            if (this.config.append_items == true) {
                blocks[i].append = true;
            }
            
        }

        return blocks;
    }


    /**
     * 
     * Parse single playlist for CSV format. Lines go in, block with items comes out
     * 
     * @param lines
     * @param filename
     * @returns {{start: boolean, items: *}}
     */
    parsePlayListCSV(lines, filename) {
        this.filename = filename;
        this.globalDate = this.getGlobalDate();

        var items = this.parseHourCSV(lines);

        var date = false;
        if (!this.globalDate) {
            //if global date is not set, get hour from first item
            if (typeof items[0].start == "undefined") {
                throw Error("Unable to parse playlist, no valid global date found and no start in first item.");
            }

            let start = moment(items[0].start);
            date = moment(start.format("YYYY-MM-DD HH")+":00:00");
        } else {
            date = this.globalDate;
            if (typeof items[0].start != "undefined") {
                let start = moment(items[0].start);
                date = moment(date.format("YYYY-MM-DD")+" "+start.format("HH")+":00:00");
            }
        }

        var block = {'start': date, 'items': items};

        if (this.config.append_items == true) {
            blocks.append = true;
        }
        
        return block;
        
    }


    parseMultiBlockCSV(lines, filename) {

        this.filename = filename;
        this.globalDate = this.getGlobalDate();
        
        var items = this.parseHourCSV(lines);
        
        var blocks = [];

        for (var i in items) {
            let item = items[i]
            
            if (typeof this.config.multi_block == "undefined" || this.config.multi_block != 'key') {
                //this is a playlist-day, but no multi-block block key has been defined
                //the parser has saved the hour from the raw date string as block identifier
                var block_id = item['_rawhour_block']
                delete item['_rawhour_block'];
            } else {
                var block_id = item[this.config.blocks_key];
            }

            if (typeof blocks[block_id] == 'undefined') {
                let startTime = this.globalDate.clone();
                startTime.add(block_id, 'hours');
                blocks[block_id] = {'start': startTime.format(), 'items': []};
            }

            blocks[block_id].items.push(item);
        
        };


        //fix the durations afterwards (if not present), less complicated
        for (var i in blocks) {
            let items = blocks[i].items;
            for (var j in items) {
                if (items[j].duration < 0) {
                    items[j].duration = 10; //default duration
                    if (j > 0) {
                        items[j - 1].duration = items[j]._start_seconds - items[j - 1]._start_seconds;
                    }
                }
            }
            for (var j in items) {
                delete items[j]['_start_seconds'];
                delete items[j]['start'];
            }

            if (this.config.append_items == true) {
                blocks[i].append = true;
            }
        }
        
        return blocks;
        
    }
        
    


    /**
     * 
     * Parse an hour/block CSV format
     * 
     * @param lines
     * @returns {Array}
     */
    parseHourCSV(lines) {

        var items = [];

        var skip = 0;
        if (typeof this.config.skip_lines != undefined &&  this.config.skip_lines) {
            skip = this.config.skip_lines;
        }

        for(var i=skip; i < lines.length; i++) {
            var line = lines[i];
            var row = {};

            for (var member in this.config.items[0].data_field_indexes) {
                var column_index = this.config.items[0].data_field_indexes[member];
                row[column_index] = line[column_index-1];
            }

            var item = this.parseItemColumned(row)
            if (item) {
                items.push(item);
            }

        }
        
        //fix the durations afterwards (if not present), less complicated
        for (var j in items) {
            if (items[j].duration < 0) {
                items[j].duration = 10; //default duration
                if (j > 0) {
                    items[j - 1].duration = items[j]._start_seconds - items[j - 1]._start_seconds;
                }
            }
        }
        for (var j in items) {
            delete items[j]['_start_seconds'];
        }

        return items;


    }

    /**
    * Parse a columned row and turn it into an Item
     */
    
    parseItemColumned(row) {

        var item = {};
        for (var typeId in this.config.items) {
            var itemType = this.config.items[typeId];

            var modelTypeColumn = itemType.data_field_indexes[itemType.model_type_field];

            if (typeof row[modelTypeColumn] == "undefined") {
                throw Error("Missing field in item: "+itemType.model_type_field+". Note that with Fixed Width or CSV all item types should have same fields! ");
            }
            var value = row[modelTypeColumn];

            if (this.matchModelType(value, itemType)) {

                //modeltype found, fill the item with the data from the columns
                item['model_type_id'] = itemType.id;
                for(var field in itemType.data_field_indexes) {
                    item[field] = row[itemType.data_field_indexes[field]];
                }

                if (this.config.type == "playlist-day" && (typeof this.config.multi_block == "undefined" || this.config.multi_block != 'key')) {
                    //this is playlist-day, get the block from the (raw hour) string
                    //acutally Powergold specific for ASC, but could come in handy in other formats
                    if (typeof item['start'] == undefined) {
                        throw Error("Sending a playlist by day, but no way to identify which block an item belongs to. Make sure a block key is defined or that the items have a 'start' field.")
                    }

                    try {
                        item['_rawhour_block'] = parseInt(item['start'].substring(0, 2));
                    } catch (e) {
                        throw Error("Sending a playlist by day, but no block key is defined. RmConnect now expects the first two digits of 'start' to identify the block");
                    }

                }


                //check if duration is defined, else extract it from start times
                if (typeof item['duration'] == "undefined") {
                    //prepare _start_seconds variable to calculate duration later
                    var startSeconds = 0;
                    if (typeof item['_rawhour_block'] != "undefined") {
                        //block per rawhour? don't try to pasre the date, assume a HH:mm:ss and extract mm and ss, Powergold specific
                        startSeconds = parseInt(item['start'].substring(3, 5)) * 60 + parseInt(item['start'].substring(6, 8));
                    } else {
                        var start = this.parseDateTime(item['start']);
                        startSeconds = 60 * start.minutes() + start.seconds();
                    }

                    item['_start_seconds'] = startSeconds;
                    item['duration'] = -1; //create for later

                    //forget about the starttime, unless this is not a playlist but that's never tested!
                    if(this.config.type != "now-on-air") {
                        delete item['start'];
                    }

                }
                
                try {
                    return this.processItem(item, itemType);
                } catch (e) {
                    this.emitWarning("Unable to process Item : "+e.message);
                }
            }
        }
    }
    

}